@@ -25,8 +25,7 @@ DEFAULT_PORT = 443
 if typing.TYPE_CHECKING:
     from .telegramclient import TelegramClient
 
-__default_log__ = logging.getLogger(__base_name__)
-__default_log__.addHandler(logging.NullHandler())
+_base_log = logging.getLogger(__base_name__)
 
 
 # In seconds, how long to wait before disconnecting a exported sender.
@@ -90,7 +89,7 @@ class TelegramBaseClient(abc.ABC):
             The API ID you obtained from https://my.telegram.org.
 
         api_hash (`str`):
-            The API ID you obtained from https://my.telegram.org.
+            The API hash you obtained from https://my.telegram.org.
 
         connection (`telethon.network.connection.common.Connection`, optional):
             The connection instance to be used when creating a new connection
@@ -200,6 +199,15 @@ class TelegramBaseClient(abc.ABC):
             If a `str` is given, it'll be passed to `logging.getLogger()`. If a
             `logging.Logger` is given, it'll be used directly. If something
             else or nothing is given, the default logger will be used.
+
+        receive_updates (`bool`, optional):
+            Whether the client will receive updates or not. By default, updates
+            will be received from Telegram as they occur.
+
+            Turning this off means that Telegram will not send updates at all
+            so event handlers, conversations, and QR login will not work.
+            However, certain scripts don't need updates, so this will reduce
+            the amount of bandwidth used.
     """
 
     # Current TelegramClient version
@@ -235,19 +243,20 @@ class TelegramBaseClient(abc.ABC):
             lang_code: str = 'en',
             system_lang_code: str = 'en',
             loop: asyncio.AbstractEventLoop = None,
-            base_logger: typing.Union[str, logging.Logger] = None):
+            base_logger: typing.Union[str, logging.Logger] = None,
+            receive_updates: bool = True
+    ):
         if not api_id or not api_hash:
             raise ValueError(
                 "Your API ID or Hash cannot be empty or None. "
                 "Refer to telethon.rtfd.io for more information.")
 
         self._use_ipv6 = use_ipv6
-        self._loop = asyncio.get_event_loop()
 
         if isinstance(base_logger, str):
             base_logger = logging.getLogger(base_logger)
         elif not isinstance(base_logger, logging.Logger):
-            base_logger = __default_log__
+            base_logger = _base_log
 
         class _Loggers(dict):
             def __missing__(self, key):
@@ -308,14 +317,14 @@ class TelegramBaseClient(abc.ABC):
         # TODO A better fix is obviously avoiding the use of `sock_connect`
         #
         # See https://github.com/LonamiWebs/Telethon/issues/1337 for details.
-        if not callable(getattr(self._loop, 'sock_connect', None)):
+        if not callable(getattr(self.loop, 'sock_connect', None)):
             raise TypeError(
                 'Event loop of type {} lacks `sock_connect`, which is needed to use proxies.\n\n'
                 'Change the event loop in use to use proxies:\n'
                 '# https://github.com/LonamiWebs/Telethon/issues/1337\n'
                 'import asyncio\n'
                 'asyncio.set_event_loop(asyncio.SelectorEventLoop())'.format(
-                    self._loop.__class__.__name__
+                    self.loop.__class__.__name__
                 )
             )
 
@@ -390,6 +399,7 @@ class TelegramBaseClient(abc.ABC):
         self._updates_handle = None
         self._last_request = time.time()
         self._channel_pts = {}
+        self._no_updates = not receive_updates
 
         if sequential_updates:
             self._updates_queue = asyncio.Queue()
@@ -402,10 +412,7 @@ class TelegramBaseClient(abc.ABC):
 
         self._authorized = None  # None = unknown, False = no, True = yes
 
-        # Update state (for catching up after a disconnection)
-        # TODO Get state from channels too
-        self._state_cache = StateCache(
-            self.session.get_update_state(0), self._log)
+        self._state_cache = StateCache(None, self._log)
 
         # Some further state for subclasses
         self._event_builders = []
@@ -459,7 +466,7 @@ class TelegramBaseClient(abc.ABC):
                 # Join the task (wait for it to complete)
                 await task
         """
-        return self._loop
+        return asyncio.get_event_loop()
 
     @property
     def disconnected(self: 'TelegramClient') -> asyncio.Future:
@@ -512,6 +519,16 @@ class TelegramBaseClient(abc.ABC):
                 except OSError:
                     print('Failed to connect')
         """
+        # Workaround specific to SQLiteSession, which sometimes need to persist info after init.
+        # Since .save() is now async we can't do that in init. Instead we do it in the first connect.
+        if isinstance(self.session, SQLiteSession) and not self.session._init_saved:
+            await self.session.save()
+            self.session._init_saved = True
+        # Update state (for catching up after a disconnection)
+        # TODO Get state from channels too
+        self._state_cache = StateCache(
+            await self.session.get_update_state(0), self._log)
+
         if not await self._sender.connect(self._connection(
             self.session.server_address,
             self.session.port,
@@ -524,7 +541,7 @@ class TelegramBaseClient(abc.ABC):
             return
 
         self.session.auth_key = self._sender.auth_key
-        self.session.save()
+        await self.session.save()
 
         self._init_request.query = functions.help.GetConfigRequest()
 
@@ -532,7 +549,7 @@ class TelegramBaseClient(abc.ABC):
             LAYER, self._init_request
         ))
 
-        self._updates_handle = self._loop.create_task(self._update_loop())
+        self._updates_handle = self.loop.create_task(self._update_loop())
 
     def is_connected(self: 'TelegramClient') -> bool:
         """
@@ -563,11 +580,11 @@ class TelegramBaseClient(abc.ABC):
                 # You don't need to use this if you used "with client"
                 await client.disconnect()
         """
-        if self._loop.is_running():
+        if self.loop.is_running():
             return self._disconnect_coro()
         else:
             try:
-                self._loop.run_until_complete(self._disconnect_coro())
+                self.loop.run_until_complete(self._disconnect_coro())
             except RuntimeError:
                 # Python 3.5.x complains when called from
                 # `__aexit__` and there were pending updates with:
@@ -611,10 +628,16 @@ class TelegramBaseClient(abc.ABC):
         # Also clean-up all exported senders because we're done with them
         async with self._borrow_sender_lock:
             for state, sender in self._borrowed_senders.values():
-                if state.should_disconnect():
-                    # disconnect should never raise
-                    await sender.disconnect()
+                # Note that we're not checking for `state.should_disconnect()`.
+                # If the user wants to disconnect the client, ALL connections
+                # to Telegram (including exported senders) should be closed.
+                #
+                # Disconnect should never raise, so there's no try/except.
+                await sender.disconnect()
+                # Can't use `mark_disconnected` because it may be borrowed.
+                state._connected = False
 
+            # If any was borrowed
             self._borrowed_senders.clear()
 
         # trio's nurseries would handle this for us, but this is asyncio.
@@ -628,7 +651,7 @@ class TelegramBaseClient(abc.ABC):
 
         pts, date = self._state_cache[None]
         if pts and date:
-            self.session.set_update_state(0, types.updates.State(
+            await self.session.set_update_state(0, types.updates.State(
                 pts=pts,
                 qts=0,
                 date=date,
@@ -636,7 +659,7 @@ class TelegramBaseClient(abc.ABC):
                 unread_count=0
             ))
 
-        self.session.close()
+        await self.session.close()
 
     async def _disconnect(self: 'TelegramClient'):
         """
@@ -661,17 +684,17 @@ class TelegramBaseClient(abc.ABC):
         # so it's not valid anymore. Set to None to force recreating it.
         self._sender.auth_key.key = None
         self.session.auth_key = None
-        self.session.save()
+        await self.session.save()
         await self._disconnect()
         return await self.connect()
 
-    def _auth_key_callback(self: 'TelegramClient', auth_key):
+    async def _auth_key_callback(self: 'TelegramClient', auth_key):
         """
         Callback from the sender whenever it needed to generate a
         new authorization key. This means we are not authorized.
         """
         self.session.auth_key = auth_key
-        self.session.save()
+        await self.session.save()
 
     # endregion
 
@@ -688,11 +711,21 @@ class TelegramBaseClient(abc.ABC):
             for pk in cls._cdn_config.public_keys:
                 rsa.add_key(pk.public_key)
 
-        return next(
-            dc for dc in cls._config.dc_options
-            if dc.id == dc_id
-            and bool(dc.ipv6) == self._use_ipv6 and bool(dc.cdn) == cdn
-        )
+        try:
+            return next(
+                dc for dc in cls._config.dc_options
+                if dc.id == dc_id
+                and bool(dc.ipv6) == self._use_ipv6 and bool(dc.cdn) == cdn
+            )
+        except StopIteration:
+            self._log[__name__].warning(
+                'Failed to get DC %s (cdn = %s) with use_ipv6 = %s; retrying ignoring IPv6 check',
+                dc_id, cdn, self._use_ipv6
+            )
+            return next(
+                dc for dc in cls._config.dc_options
+                if dc.id == dc_id and bool(dc.cdn) == cdn
+            )
 
     async def _create_exported_sender(self: 'TelegramClient', dc_id):
         """
@@ -786,7 +819,7 @@ class TelegramBaseClient(abc.ABC):
         if not session:
             dc = await self._get_dc(cdn_redirect.dc_id, cdn=True)
             session = self.session.clone()
-            await session.set_dc(dc.id, dc.ip_address, dc.port)
+            session.set_dc(dc.id, dc.ip_address, dc.port)
             self._exported_sessions[cdn_redirect.dc_id] = session
 
         self._log[__name__].info('Creating new CDN client')
@@ -823,6 +856,11 @@ class TelegramBaseClient(abc.ABC):
                 executed sequentially on the server. They run in arbitrary
                 order by default.
 
+            flood_sleep_threshold (`int` | `None`, optional):
+                The flood sleep threshold to use for this request. This overrides
+                the default value stored in
+                `client.flood_sleep_threshold <telethon.client.telegrambaseclient.TelegramBaseClient.flood_sleep_threshold>`
+
         Returns:
             The result of the request (often a `TLObject`) or a list of
             results if more than one request was given.
